# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'login.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(246, 140)
        self.verticalLayoutWidget = QtWidgets.QWidget(Form)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(9, 9, 222, 116))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_info = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_info.setAlignment(QtCore.Qt.AlignCenter)
        self.label_info.setObjectName("label_info")
        self.verticalLayout.addWidget(self.label_info)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_username = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_username.setObjectName("label_username")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_username)
        self.edit_username = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.edit_username.setObjectName("edit_username")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.edit_username)
        self.label_password = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_password.setObjectName("label_password")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_password)
        self.edit_password = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.edit_password.setObjectName("edit_password")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.edit_password)
        self.verticalLayout.addLayout(self.formLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.btn_login = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.btn_login.setObjectName("btn_login")
        self.horizontalLayout_2.addWidget(self.btn_login)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Log in"))
        self.label_info.setText(_translate("Form", "Please, provide your credentials."))
        self.label_username.setText(_translate("Form", "Username"))
        self.label_password.setText(_translate("Form", "Password"))
        self.btn_login.setText(_translate("Form", "Login"))



import time
import json
import logging
from datetime import datetime
import locale

from .variables import BYTES_RECV_SIZE


logger = logging.getLogger('server')


def send_message(sock, message: dict, encoding='ascii'):
    if not isinstance(message, dict):
        raise TypeError
    message = json.dumps(message)
    sock.send(message.encode(encoding))


def get_message(sock, encoding='ascii') -> dict:
    response = sock.recv(BYTES_RECV_SIZE)
    if isinstance(response, bytes):
        json_response = response.decode(encoding)
        if json_response:
            response = json.loads(json_response)
            if isinstance(response, dict):
                return response
            raise ValueError
        raise ValueError
    raise ValueError


def get_unix_time_str():
    unix_time_str = time.ctime(time.time()) + '\n'
    return unix_time_str


def get_datetime_from_unix_str(unix_str: str):
    locale.setlocale(locale.LC_TIME, 'en_US.utf8')
    return datetime.strptime(unix_str.strip(), "%a %b %d %H:%M:%S %Y")



import dis


class ClientVerifier(type):
    """Metaclass for verifying if client is correct"""
    def __init__(cls, clsname, bases, clsdict):

        presence_get_message = False
        presence_send_message = False

        for func in clsdict.keys():
            ret = dis.get_instructions(clsdict[func])

            for i in ret:
                if i.opname == 'LOAD_GLOBAL':
                    if i.argval in ('accept', 'listen', 'socket'):
                        raise TypeError('You can\'t use accept/listen/socket in Client!')
                    if i.argval == 'get_message':
                        presence_get_message = True
                    elif i.argval == 'send_message':
                        presence_send_message = True
        if not any((presence_send_message, presence_get_message)):
            raise TypeError('Client has no get_message and send_message methods!')

        super().__init__(clsname, bases, clsdict)


class ServerVerifier(type):
    """Metaclass for verifying if server is correct"""
    def __init__(cls, clsname, bases, clsdict):

        methods_1 = []
        methods_2 = []
        attrs = []

        for func in clsdict.keys():
            try:
                ret = dis.get_instructions(clsdict[func])
            except TypeError as e:
                pass
            else:
                for i in ret:
                    if i.opname == 'LOAD_GLOBAL':
                        if i.argval not in methods_1:
                            methods_1.append(i.argval)
                    elif i.opname == 'LOAD_METHOD':
                        if i.argval not in methods_2:
                            methods_2.append(i.argval)
                    elif i.opname == 'LOAD_ATTR':
                        if i.argval not in attrs:
                            attrs.append(i.argval)

        print(methods_1)
        print(methods_2)
        print(attrs)

        all_methods = []
        all_methods.extend(methods_1)
        all_methods.extend(methods_2)

        if 'connect' in all_methods:
            raise TypeError('You can\'t use connect in server methods')
        if not ('SOCK_STREAM' in attrs or 'SOCK_STREAM' in all_methods and 'AF_INET' in attrs or 'AF_INET' in all_methods):
            raise TypeError('Incorrect initialization of socket. Use SOCK_STREAM and AF_INET')

        super().__init__(clsname, bases, clsdict)



from sqlalchemy import create_engine, Column, Integer, String, DateTime, ForeignKey, Text
from sqlalchemy.orm import declarative_base, sessionmaker
from datetime import datetime
import os

Base = declarative_base()


class ClientStorage:
    """
    Class that contains ORM classes for DB tables
    """
    class Contact(Base):
        __tablename__ = 'contacts'
        id = Column(Integer, primary_key=True)
        username = Column(String, unique=True, nullable=False)

        def __init__(self, username):
            self.username = username

        def __repr__(self):
            if self.username:
                return self.username
            else:
                return 'WTF?'

    class Message(Base):
        __tablename__ = 'messages'
        id = Column(Integer, primary_key=True)
        from_ = Column(Integer, ForeignKey('contacts.id'), nullable=True)
        to = Column(Integer, ForeignKey('contacts.id'), nullable=True)
        timestamp = Column(DateTime)
        text = Column(Text)

        def __init__(self, text: str, timestamp, from_=None, to=None):
            # to=None means to self, from_=None means from self
            self.text = text
            self.timestamp = timestamp
            self.from_ = from_
            self.to = to

        def __repr__(self):
            return self.text

    def __init__(self, username: str):
        self.username = username
        current_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), f'{username}_storage.db')
        self.engine = create_engine(f'sqlite:///{current_dir}', echo=False, pool_recycle=7200,
                                    connect_args={'check_same_thread': False})
        Base.metadata.create_all(self.engine)
        Session = sessionmaker()
        self.session = Session(bind=self.engine)

    def get_history(self, current_chat):
        """Returns messages history for user and current chat"""
        messages_from = self.session.query(self.Message).filter_by(from_=current_chat)
        messages_to = self.session.query(self.Message).filter_by(to=current_chat)
        messages_from = [(m.from_, m.to, m.timestamp, m.text) for m in messages_from]
        messages_to = [(m.from_, m.to, m.timestamp, m.text) for m in messages_to]
        print([i[2] for i in messages_to])
        history = sorted([*messages_from, *messages_to], key=lambda x: x[2])  # datetime.strptime(x[2],
        return history

    def get_contacts(self):
        """Returns user's contacts"""
        contacts = self.session.query(self.Contact).all()
        return [contact.username for contact in contacts]

    def is_contact_exists(self, username: str):
        """Checks if user is in the contacts list"""
        contact_exists = self.session.query(self.Contact).filter_by(username=username).first()
        return contact_exists

    def add_contact(self, username: str):
        """Add user to contacts list"""
        contact = self.Contact(username=username)
        self.session.add(contact)
        self.session.commit()

    def delete_contact(self, username: str):
        """Delete user from contacts list"""
        self.session.query(self.Contact).filter_by(username=username).delete()
        self.session.commit()

    def create_message(self, text: str, timestamp, from_=None, to=None):
        """Create JIM-message for sending"""
        if from_:
            if not self.is_contact_exists(from_) and from_ != to and self.username != from_:
                self.add_contact(from_)
        if to:
            if not self.is_contact_exists(to) and from_ != to and self.username != to:
                self.add_contact(to)
        message = self.Message(text, timestamp, from_, to)
        self.session.add(message)
        self.session.commit()

    def get_messages(self):
        """Returns all messages from DB"""
        return self.session.query(self.Message).all()

    def delete_all_contacts(self):
        """Deletes all user's contacts"""
        self.session.query(self.Contact).delete()
        self.session.commit()

    def delete_all_messages(self):
        """Deletes all messages"""
        self.session.query(self.Message).delete()
        self.session.commit()


if __name__ == '__main__':
    storage = ClientStorage('guest')
    storage.delete_all_contacts()
    storage.delete_all_messages()
    print('Empty', storage.get_contacts())
    storage.add_contact('test_1')
    print('Added 1', storage.get_contacts())
    storage.add_contact('test_1')
    print('Added the same', storage.get_contacts())
    storage.delete_contact('test_1')
    print('Deleted 1', storage.get_contacts())
    storage.create_message('Hello, test_1!', datetime.now(), 'test_2', 'test_1')
    print('test_2 -> test_1', storage.get_messages())
    storage.create_message('Hello, test_2!', datetime.now(), 'test_1', 'test_2')
    print('test_1 -> test_2', storage.get_messages())



from threading import Thread, Lock
import logging
import json
import socket
import sys
import os
import time

from PyQt5.QtCore import QObject, pyqtSignal

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from common.variables import PORT, ACT_PRESENCE, HOST, ACT_MESSAGE, ACT_ADD_CONTACT, ACT_DEL_CONTACT, ACT_GET_CONTACTS, \
    ACT_EXIT, ACT_GET_CLIENTS
from common.utils import get_unix_time_str, send_message, get_message, get_datetime_from_unix_str
import log.client_log_config

logger = logging.getLogger('client')
socket_lock = Lock()


class ClientTransport(Thread, QObject):
    """Class for interaction with DB"""
    new_message = pyqtSignal(str)
    connection_lost = pyqtSignal()

    def __init__(self, host, port, database, name, password):
        Thread.__init__(self)
        QObject.__init__(self)

        self.database = database
        self.account_name = name
        self.password = password
        self.transport = None
        self.connection_init(port, host)

        self.running = True

    def run(self) -> None:
        """Main loop for this class. Getting messages and handle them"""
        logger.info('Getting new messages...')
        while self.running:
            # Отдыхаем секунду и снова пробуем захватить сокет. Если не сделать тут задержку,
            # то отправка может достаточно долго ждать освобождения сокета.
            time.sleep(1)
            with socket_lock:
                try:
                    self.transport.settimeout(0.5)
                    message = get_message(self.transport)
                except ValueError:
                    pass
                except OSError as err:
                    if err.errno:
                        # выход по таймауту вернёт номер ошибки err.errno равный None
                        # поэтому, при выходе по таймауту мы сюда попросту не попадём
                        logger.critical(f'Lost connection to a server.')
                        self.running = False
                        self.connection_lost.emit()
                # Проблемы с соединением
                except (ConnectionError, ConnectionAbortedError,
                        ConnectionResetError, json.JSONDecodeError, TypeError):
                    logger.debug(f'Lost connection to a server.')
                    self.running = False
                    self.connection_lost.emit()
                # Если сообщение получено, то вызываем функцию обработчик:
                else:
                    logger.debug(f'Got new message from a server: {message}')
                    self.handle_answer(message)
                finally:
                    self.transport.settimeout(5)

    def shutdown(self):
        """Gracefully closes connections"""
        self.running = False
        with socket_lock:
            try:
                send_message(self.transport, self.create_exit_message())
            except OSError:
                pass
        logger.info('Exiting...')
        time.sleep(0.5)

    def connection_init(self, port, ip):
        """Trying to connect to a server"""
        self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.transport.settimeout(5)
        connected = False
        for i in range(5):
            logger.info(f'Trying to connect №{i + 1}/5')
            try:
                self.transport.connect((ip, port))
            except (OSError, ConnectionRefusedError):
                pass
            else:
                connected = True
                break
            time.sleep(1)

        if not connected:
            logger.critical('Can\'t connect to a server.')
            raise ValueError('Can\'t connect to a server.')

        logger.debug('Connected to a server!')

        try:
            with socket_lock:
                send_message(self.transport, self.create_presence())
                self.handle_answer(get_message(self.transport))
        except (OSError, json.JSONDecodeError):
            logger.critical('Lost connection to a server.')
            raise ValueError('Lost connection to a server.')

        logger.info('Connected to a server!')

    def get_clients(self):
        """Returns clients list from a server"""
        with socket_lock:
            send_message(self.transport, self.create_get_clients_message())
            return self.handle_answer(get_message(self.transport))

    def add_contact(self, user):
        """Adds user to a contacts list"""
        if not self.database.is_contact_exists(user):
            self.database.add_contact(user)
            with socket_lock:
                send_message(self.transport, self.create_contact_edit_message(ACT_ADD_CONTACT, user))
                self.handle_answer(get_message(self.transport))
            print(f'You \'ve added {user} to friends!')
        else:
            print(f'You can\'t add {user} to friends!')

    def del_contact(self, user):
        """Deletes user from a contacts list"""
        if self.database.is_contact_exists(user):
            self.database.delete_contact(user)
            with socket_lock:
                send_message(self.transport, self.create_contact_edit_message(ACT_DEL_CONTACT, user))
                self.handle_answer(get_message(self.transport))
        else:
            print(f'You can\'t delete {user} from friends!')

    def send_message_to_user(self, username, text):
        """Sends message to a user"""
        with socket_lock:
            send_message(self.transport, self.create_message(text, username, self.account_name))
            self.handle_answer(get_message(self.transport))

    def handle_answer(self, answer):
        """Handle answer depending on the context"""
        try:
            if 'action' in answer and answer['action'] == ACT_MESSAGE and 'user' in answer and 'to' in answer and \
                    answer['to'] == self.account_name:
                print(f'NEW | From {answer["user"]["account_name"]} | {answer["msg"]}')
                text = answer['msg']
                timestamp = get_datetime_from_unix_str(answer['time'])
                from_ = answer['user']['account_name']
                to_ = answer['to']
                self.database.create_message(text, timestamp, from_, to_)
                self.new_message.emit(from_)
            elif 'action' in answer and answer['action'] == ACT_MESSAGE and 'user' in answer and 'to' in answer and \
                    answer['user']['account_name'] == self.account_name:
                text = answer['msg']
                timestamp = get_datetime_from_unix_str(answer['time'])
                from_ = answer['user']['account_name']
                to_ = answer['to']
                self.database.create_message(text, timestamp, from_, to_)
            elif 'action' in answer and answer['action'] == ACT_EXIT and 'user' in answer and answer['user'] == self.account_name:
                return
            elif 'action' in answer and answer['action'] == ACT_PRESENCE and 'user' in answer:
                logger.info('Status: Online!')
            elif 'action' in answer and answer['action'] == ACT_GET_CLIENTS and 'user' in answer and 'response' \
                    in answer and answer['response'] == 200:
                return answer['clients']
            elif 'action' in answer and answer['action'] == ACT_GET_CONTACTS and 'response' in answer\
                    and answer['response'] == 202:
                contacts = self.storage.get_contacts()
                print(f'Your contacts: {", ".join(contacts)}')
            elif 'action' in answer and answer['action'] == ACT_ADD_CONTACT and 'response' in answer \
                    and 'user_id' in answer:
                pass
            elif 'action' in answer and answer['action'] == ACT_DEL_CONTACT and 'response' in answer\
                    and 'user_id' in answer:
                pass
            elif "error" in answer:
                logger.error(answer['error'])
                exit(1)
            else:
                logger.error(answer)
                exit(1)
        except ValueError as e:
            logger.exception(e)

    def create_get_clients_message(self):
        message = {
            "action": ACT_GET_CLIENTS,
            "time": get_unix_time_str(),
            "type": "status",
            "user": {
                "account_name": self.account_name,
                "status": "online"
            }
        }
        return message

    def create_presence(self):
        presence = {
            "action": ACT_PRESENCE,
            "time": get_unix_time_str(),
            "type": "status",
            "user": {
                "account_name": self.account_name,
                "hashed_password": self.password,
                "status": "online"
            }
        }
        return presence

    def create_exit_message(self):
        return {
            'action': 'exit',
            'time': time.time(),
            'user': {
                'account_name': self.account_name,
                'status': 'offline'
            }
        }

    def create_contact_edit_message(self, action, username):
        message_dict = {
            "action": action,
            "time": get_unix_time_str(),
            "user_id": username,
            "user_login": self.account_name
        }
        return message_dict

    def create_get_contacts_message(self):
        message_dict = {
            "action": ACT_GET_CONTACTS,
            "time": get_unix_time_str(),
            "user_login": self.account_name
        }
        return message_dict

    def create_message(self, message: str, to: str, account_name: str = 'guest'):
        message_dict = {
            "action": ACT_MESSAGE,
            "time": get_unix_time_str(),
            "type": "status",
            "user": {
                "account_name": account_name,
                "status": "online"
            },
            "to": to,
            "msg": message,
        }
        logger.info(f'Created message {account_name} -> {to} | {message}')
        return message_dict

    def print_help(self):
        print('Type: m - start a new message, a - add contact, d - delete contact, c - print contacts, q - quit\n')



import json
from unittest import TestCase, main
import sys
import os

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import ENCODING
from common.utils import send_message, get_message, get_unix_time_str


class TestSocket:
    def __init__(self, test_dict):
        self.test_dict = test_dict
        self.encoded_message = None
        self.received_message = None

    def send(self, message: dict):
        json_text_message = json.dumps(self.test_dict)
        self.encoded_message = json_text_message.encode(ENCODING)
        self.received_message = message

    def recv(self, max_bytes):
        json_text_message = json.dumps(self.test_dict)
        return json_text_message.encode(ENCODING)


class TestUtils(TestCase):
    test_dict_send = {
        "action": "presence",
        "time": 1,
        "type": "status",
        "user": {
            "account_name": "Not a guest",
            "status": "online"
        }
    }

    def test_send_message_ok(self):
        test_socket = TestSocket(self.test_dict_send)
        send_message(test_socket, self.test_dict_send)
        self.assertEqual(test_socket.encoded_message, test_socket.received_message)

    def test_send_message_type_error(self):
        test_socket = TestSocket(self.test_dict_send)
        send_message(test_socket, self.test_dict_send)
        self.assertRaises(TypeError, send_message, test_socket, "wrong_dict")

    def test_get_message(self):
        test_socket = TestSocket(self.test_dict_send)
        received_message = get_message(test_socket)
        self.assertEqual(received_message, self.test_dict_send)


if __name__ == '__main__':
    main()



HOST = 'localhost'
PORT = 7777

ENCODING = 'ascii'

BYTES_RECV_SIZE = 1000000
ACT_PRESENCE = 'presence'
ACT_PROBE = 'probe'
ACT_MESSAGE = 'msg'
ACT_ADD_CONTACT = 'add_contact'
ACT_DEL_CONTACT = 'del_contact'
ACT_GET_CONTACTS = 'get_contacts'
ACT_QUIT = 'quit'
ACT_AUTHENTICATE = 'authenticate'
ACT_JOIN = 'join'
ACT_LEAVE = 'leave'
ACT_EXIT = 'exit'
ACT_GET_CLIENTS = 'get_clients'

RESP_BASE_NOTIF = 100
RESP_IMORTANT_NOTIF = 101

RESP_OK = 200
RESP_OBJ_CREATED = 201
RESP_ACCEPTED = 202

RESP_WRONG_REQUEST = 400
RESP_NOT_AUTHORIZED = 401
RESP_WRONG_CREDENTIALS = 402
RESP_FORBIDDEN = 403
RESP_NOT_FOUND = 404
RESP_CONFLICT = 409
RESP_GONE = 410

RESP_BAD_GATEWAY = 500



class Port:
    """Descriptor for port object"""

    def __init__(self):
        self._value = 7777
        print('init')

    def __get__(self, instance, instance_type):
        print('get')
        return self._value

    def __set__(self, instance, value):
        print('set')
        if not (1024 <= value < 65536):
            raise ValueError('Please, specify safe port (1024-65536)')
        self._value = value




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client_chat.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(640, 480)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_contacts = QtWidgets.QLabel(self.centralwidget)
        self.label_contacts.setAlignment(QtCore.Qt.AlignCenter)
        self.label_contacts.setObjectName("label_contacts")
        self.gridLayout_2.addWidget(self.label_contacts, 0, 0, 1, 1)
        self.edit_message = QtWidgets.QLineEdit(self.centralwidget)
        self.edit_message.setObjectName("edit_message")
        self.gridLayout_2.addWidget(self.edit_message, 4, 1, 1, 1)
        self.btn_send_message = QtWidgets.QPushButton(self.centralwidget)
        self.btn_send_message.setObjectName("btn_send_message")
        self.gridLayout_2.addWidget(self.btn_send_message, 5, 1, 1, 1)
        self.label_messages = QtWidgets.QLabel(self.centralwidget)
        self.label_messages.setAlignment(QtCore.Qt.AlignCenter)
        self.label_messages.setObjectName("label_messages")
        self.gridLayout_2.addWidget(self.label_messages, 0, 1, 1, 1)
        self.list_contacts = QtWidgets.QListView(self.centralwidget)
        self.list_contacts.setObjectName("list_contacts")
        self.gridLayout_2.addWidget(self.list_contacts, 2, 0, 3, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_add_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_add_contact.setObjectName("btn_add_contact")
        self.horizontalLayout.addWidget(self.btn_add_contact)
        self.btn_del_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_del_contact.setObjectName("btn_del_contact")
        self.horizontalLayout.addWidget(self.btn_del_contact)
        self.horizontalLayout_5.addLayout(self.horizontalLayout)
        self.gridLayout_2.addLayout(self.horizontalLayout_5, 5, 0, 1, 1)
        self.label_help = QtWidgets.QLabel(self.centralwidget)
        self.label_help.setObjectName("label_help")
        self.gridLayout_2.addWidget(self.label_help, 3, 1, 1, 1)
        self.list_messages = QtWidgets.QListView(self.centralwidget)
        self.list_messages.setObjectName("list_messages")
        self.gridLayout_2.addWidget(self.list_messages, 2, 1, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 640, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Messenger"))
        self.label_contacts.setText(_translate("MainWindow", "Contacts"))
        self.btn_send_message.setText(_translate("MainWindow", "Send"))
        self.label_messages.setText(_translate("MainWindow", "Messages"))
        self.btn_add_contact.setText(_translate("MainWindow", "Add contact"))
        self.btn_del_contact.setText(_translate("MainWindow", "Delete contact"))
        self.label_help.setText(_translate("MainWindow", "Type text below and click \"Send\""))



import logging
from logging.handlers import TimedRotatingFileHandler
import os


format = logging.Formatter('%(asctime)-10s %(levelname)s %(module)s %(message)s')
current_directory = os.path.dirname(os.path.abspath(__file__))
log_path = os.path.join(current_directory, 'files')
log_filename = os.path.join(log_path, 'server.log')

handler = TimedRotatingFileHandler(filename=log_filename, encoding='utf-8', interval=1, when='D')
handler.setLevel(logging.DEBUG)
handler.setFormatter(format)
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(format)
server_logger = logging.getLogger('server')
server_logger.addHandler(handler)
server_logger.addHandler(stream_handler)
server_logger.setLevel(logging.DEBUG)


if __name__ == '__main__':
    logger = logging.getLogger('server')
    logger.debug('Testing')
    logger.info('Testing')
    logger.error('Testing')



import logging
import sys
from functools import wraps
import time
import inspect

if sys.argv[0].endswith('client.py'):
    logger = logging.getLogger('client')
else:
    logger = logging.getLogger('server')


def log(func):
    @wraps(func)
    def decorator(*args, **kwargs):
        time_1 = time.time()
        result = func(*args, **kwargs)
        time_2 = time.time()
        elapsed_time = round(time_2-time_1, 10)
        logger.debug(f'Function {func.__name__} was executed in {elapsed_time} s.')
        calling_func_name = inspect.stack()[1].function
        logger.debug(f'Function {func.__name__} was invoked from {calling_func_name}')
        return result
    return decorator



from unittest import TestCase, main
import sys
import os

sys.path.append(os.path.join(os.getcwd(), '..'))
from client import create_presence


class TestClient(TestCase):
    def test_create_presence_guest(self):
        presence = {
            "action": "presence",
            "time": 1,
            "type": "status",
            "user": {
                "account_name": "guest",
                "status": "online"
            }
        }
        answer = create_presence()
        answer["time"] = 1
        self.assertEqual(presence, answer)

    def test_create_presence_not_guest(self):
        presence = {
            "action": "presence",
            "time": 1,
            "type": "status",
            "user": {
                "account_name": "Not a guest",
                "status": "online"
            }
        }
        answer = create_presence("Not a guest")
        answer["time"] = 1
        self.assertEqual(presence, answer)


if __name__ == '__main__':
    main()



import hashlib

from .client_chat import Ui_MainWindow
from .client_add_contact import Ui_form_add_contact
from .login import Ui_Form
import sys
from PyQt5.QtWidgets import (
    QApplication, QMainWindow, QMessageBox
)
from PyQt5.QtCore import pyqtSlot, Qt, pyqtSignal
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor


class ChatGui(QMainWindow, Ui_MainWindow):

    is_authenticated = pyqtSignal()

    def __init__(self, storage, transport, parent=None):
        super().__init__(parent)
        self.storage = storage
        self.transport = transport
        self.contacts_model = None
        self.current_chat = None
        self.add_contact_gui = None
        self.history_model = None
        self.login_window = None
        self.alert = QMessageBox()
        self.setupUi(self)
        self.clients_list_update()
        self.initUi()

    def grant_send_message(self, grant: bool):
        self.btn_send_message.setEnabled(grant)
        self.edit_message.setEnabled(grant)

    def initUi(self):
        self.grant_send_message(False)
        self.list_contacts.clicked.connect(self.select_contact)
        self.btn_add_contact.clicked.connect(self.open_add_contact_gui)
        self.btn_del_contact.clicked.connect(self.del_contact)
        self.btn_send_message.clicked.connect(self.send_message)
        self.show()

    def open_add_contact_gui(self):
        all_clients = self.transport.get_clients()
        self.add_contact_gui = AddContactGui(all_clients)
        self.add_contact_gui.btn_add.clicked.connect(self.add_contact)
        self.add_contact_gui.btn_cancel.clicked.connect(self.add_contact_gui.close)

    def add_contact(self):
        item = self.add_contact_gui.select_user
        new_contact = str(item.currentText())
        self.transport.add_contact(new_contact)
        self.clients_list_update()

    def del_contact(self):
        self.transport.del_contact(self.current_chat)
        self.clients_list_update()

    def select_contact(self):
        self.current_chat = self.list_contacts.currentIndex().data()
        self.grant_send_message(True)
        self.messages_list_update()

    def send_message(self):
        message_text = self.edit_message.text()
        if not message_text:
            return
        alert = self.transport.send_message_to_user(self.current_chat, message_text)
        if alert:
            self.alert.warning(self, 'Oops', 'It seems that user is not online. Try later.')
        self.messages_list_update()
        self.edit_message.setText('')

    def messages_list_update(self):
        history = self.storage.get_history(self.current_chat)

        self.history_model = QStandardItemModel()
        self.list_messages.setModel(self.history_model)

        self.history_model.clear()
        length = len(history)
        start_index = 0

        if length > 20:
            start_index = length - 20

        for i in range(start_index, length):
            item = history[i]
            print(item)
            if item[1] != self.current_chat:
                mess = QStandardItem(f'{item[2]}\nВходящее от {item[0]}\n{item[3]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'{item[2]}\nИсходящее от {item[0]}\n{item[3]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.list_messages.scrollToBottom()

    def make_connection(self, obj):
        obj.new_message.connect(self.message)
        obj.connection_lost.connect(self.connection_lost)

    @pyqtSlot(str)
    def message(self, sender):
        self.alert.information(self, 'Info', f'New message from {sender}!')
        self.messages_list_update()
        self.clients_list_update()

    def connection_lost(self):
        self.alert.critical(self, 'Oops', 'Lost connection to a server.')

    def clients_list_update(self):
        contacts_list = self.storage.get_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contacts_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.list_contacts.setModel(self.contacts_model)


class AddContactGui(QMainWindow, Ui_form_add_contact):
    def __init__(self, clients, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.clients = clients
        self.clients_model = None
        self.initUi()

    def initUi(self):
        self.clients_list_update()
        self.show()

    def clients_list_update(self):
        print(self.clients)
        self.clients_model = QStandardItemModel()
        for i in sorted(self.clients):
            item = QStandardItem(i)
            self.clients_model.appendRow(item)
        self.select_user.setModel(self.clients_model)


class LoginGui(QMainWindow, Ui_Form):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setupUi(self)
        self.initUi()
        self.username = None
        self.password = None
        self.messages = QMessageBox()

    def initUi(self):
        self.btn_login.clicked.connect(self.authenticate)
        self.show()

    def authenticate(self):
        self.username = self.edit_username.text().strip()
        self.password = self.edit_password.text().strip()
        if not (self.username and self.password):
            self.messages.critical(self, 'Error', 'You should enter username and password both!')
        else:
            hashed_password = hashlib.pbkdf2_hmac('sha256', self.password.encode('utf-8'),
                                                  self.username.lower().encode('utf-8'), 100000)
            self.password = str(hashed_password)
            self.close()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    # win = ChatGui(None, None)
    # add_contact = AddContactGui()
    login = LoginGui()
    sys.exit(app.exec())



from unittest import TestCase, main
import sys
import os

sys.path.append(os.path.join(os.getcwd(), '..'))
from server import process_client_message


class TestServer(TestCase):
    answer_ok = {
        "response": 200
    }
    answer_wrong = {
        "response": 400,
        "error": "Bad Request"
    }

    def test_process_client_message_ok(self):
        message = {
            "action": "presence",
            "time": 1,
            "user": "guest"
        }
        self.assertEqual(process_client_message(message), self.answer_ok)

    def test_process_client_message_wrong(self):
        message = {
            "action": "nothing",
            "time": 1,
            "user": "guest"
        }
        self.assertEqual(process_client_message(message), self.answer_wrong)


if __name__ == '__main__':
    main()



import logging
import argparse
import sys
import os

from PyQt5.QtWidgets import QApplication

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from common.variables import PORT, ACT_PRESENCE, HOST, ACT_MESSAGE, ACT_ADD_CONTACT, ACT_DEL_CONTACT, ACT_GET_CONTACTS
from client_database import ClientStorage
from client_transport import ClientTransport
from ui.gui import ChatGui, LoginGui
import log.client_log_config


logger = logging.getLogger('client')


def get_args():
    """Returns Server\'s address, port and client\'s name from cmd args
    Example: python client.py 127.0.0.1 8000 -n NAME
    """
    parser = argparse.ArgumentParser(description='A client')
    parser.add_argument("address", help='Server\'s address. Default: 127.0.0.1', nargs='?', default=HOST)
    parser.add_argument("port", help='Server\'s port. Default: 7777', nargs='?', default=PORT)
    parser.add_argument('-n', dest='name', default=None, required=False)
    parser.add_argument('-p', dest='password', default=None, required=False)
    args = parser.parse_args()
    logger.debug(f'Args: {args.address}, {args.port} {args.name} {args.password}')
    return args.address, args.port, args.name, args.password


def prompt_login_password():
    """QT Window for prompting username and password from a user
    """
    login_qapp = QApplication(sys.argv)
    login = LoginGui()
    login_qapp.exec_()
    name, password = login.username, login.password
    return name, password


def main():
    """Main loop for client that obtains cmd args, prompts username and password, sets db connection and GUI"""
    host, port, name, password = get_args()
    if not (name and password):
        name, password = prompt_login_password()

    main_qapp = QApplication(sys.argv)

    while True:
        logger.info(f'Client {name} trying to connect to {host}:{port}')
        storage = ClientStorage(name)

        try:
            transport = ClientTransport(host, port, storage, name, password)
        except ValueError as e:
            logger.error(e)
            exit(1)
        else:
            break

    transport.daemon = True
    transport.start()

    chat_window = ChatGui(storage, transport)
    chat_window.make_connection(transport)

    main_qapp.exec_()

    transport.shutdown()
    transport.join()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client_add_contact.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_form_add_contact(object):
    def setupUi(self, form_add_contact):
        form_add_contact.setObjectName("form_add_contact")
        form_add_contact.resize(286, 124)
        self.widget_form = QtWidgets.QWidget(form_add_contact)
        self.widget_form.setGeometry(QtCore.QRect(9, 9, 265, 101))
        self.widget_form.setObjectName("widget_form")
        self.formLayout = QtWidgets.QFormLayout(self.widget_form)
        self.formLayout.setContentsMargins(0, 0, 0, 0)
        self.formLayout.setObjectName("formLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_add_contact = QtWidgets.QLabel(self.widget_form)
        self.label_add_contact.setAlignment(QtCore.Qt.AlignCenter)
        self.label_add_contact.setObjectName("label_add_contact")
        self.verticalLayout.addWidget(self.label_add_contact)
        self.select_user = QtWidgets.QComboBox(self.widget_form)
        self.select_user.setObjectName("select_user")
        self.verticalLayout.addWidget(self.select_user)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btn_add = QtWidgets.QPushButton(self.widget_form)
        self.btn_add.setObjectName("btn_add")
        self.horizontalLayout_3.addWidget(self.btn_add)
        self.btn_cancel = QtWidgets.QPushButton(self.widget_form)
        self.btn_cancel.setObjectName("btn_cancel")
        self.horizontalLayout_3.addWidget(self.btn_cancel)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.formLayout.setLayout(0, QtWidgets.QFormLayout.SpanningRole, self.verticalLayout)

        self.retranslateUi(form_add_contact)
        QtCore.QMetaObject.connectSlotsByName(form_add_contact)

    def retranslateUi(self, form_add_contact):
        _translate = QtCore.QCoreApplication.translate
        form_add_contact.setWindowTitle(_translate("form_add_contact", "Add contact"))
        self.label_add_contact.setText(_translate("form_add_contact", "Choose a user to add to contact list"))
        self.btn_add.setText(_translate("form_add_contact", "Add"))
        self.btn_cancel.setText(_translate("form_add_contact", "Cancel"))
